//  MODULE:   GetPortNumber.c
//
//  PURPOSE:	Returns the Port Number of the service from the
//				registry information
//
//  FUNCTIONS:
//
//		LONG GetPortNumber(WORD *wPortNumber)
//
//  COMMENTS:
//
//  AUTHOR: Shane Hebert
//


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <winsock.h>
#include "service.h"

//
//  FUNCTION: LONG GetPortNumber(WORD *wPortNumber)
//
//  PURPOSE:
//		This function retrieves the port number from
//		the registry
//
//  PARAMETERS:
//		wPortNumber	-	word location to store the port
//						number in
//
//  RETURN VALUE:
//    status
//
//  COMMENTS:
//
LONG GetPortNumber(WORD *wPortNumber)
{
	HKEY hkSoftwareSubKey;
	HKEY hkMPISubKey;
	LONG nStatus;
	DWORD dwPortNumber=0;
	LPTSTR lpszSoftwareSubKey = TEXT(SZSOFTWARESUBKEY);
	LPTSTR lpszMPISubKey = TEXT(SZMPISUBKEY);
	LPTSTR lpszKeyClass = TEXT(SZKEYCLASS);
	LPTSTR lpszVersionText = TEXT(SZVERSIONTEXT);
	LPTSTR lpszVersionNumber = TEXT(SZVERSIONNUMBER);
	LPTSTR lpszPortNumberText = TEXT(SZPORTNUMBERTEXT);
	
	//  Open HKEY_LOCAL_MACHINE
	nStatus = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			lpszSoftwareSubKey,
			0,
			KEY_ALL_ACCESS,
			&hkSoftwareSubKey);
	if (ERROR_SUCCESS == nStatus)
		//  Open MPI Server
		nStatus = RegOpenKeyEx(
					hkSoftwareSubKey,
					lpszMPISubKey,
					0,
					KEY_ALL_ACCESS,
					&hkMPISubKey);
	if (ERROR_SUCCESS == nStatus)
	{
		DWORD dwType;
		DWORD dwPortNumber;
		DWORD dwDWSize=sizeof(DWORD);

		// Ask it for the Port Number value
		nStatus = RegQueryValueEx(
					hkMPISubKey,
					lpszPortNumberText,
					0,
					&dwType,
					(BYTE*)&dwPortNumber,
					&dwDWSize);
		if (ERROR_SUCCESS == nStatus)
		{
			*wPortNumber = (WORD)dwPortNumber;
		}
	}
	return nStatus;
}
