/*
 *  $Id: mpisys.h.in,v 1.1 1994/11/29 20:32:27 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/* include various mpi-implementation only includes.  The user should not
   ever need this include (or anything that it references). */
#include "sbcnst.h"

/* 
   define MPID_NO_FORTRAN if the Fortran interface is not to be supported
   (perhaps because there is no Fortran compiler)
 */

#ifndef MPID_NO_FORTRAN
	#define MPID_NO_FORTRAN
#endif

#if defined(MPIR_DEBUG_MEM) || defined(MPIR_MEMDEBUG)
	#undef MPIR_SBinit
	#undef MPIR_SBalloc
	#undef MPIR_SBfree
	#undef MPIR_SBdestroy
#endif

/* This is the allocation unit. */
typedef struct _sbialloc {
    struct _sbialloc *next;
    int              nbytes, nballoc;
    int              nbinuse;
    } MPIR_SBiAlloc;

/* Blocks are linked together; they are (much) larger than this */
#ifdef DEBUG
typedef struct {
    long sentinal_1;
    char *next;
    long sentinal_2;
    } MPIR_SBblock;
#else
typedef struct {
    char *next;
    } MPIR_SBblock;
#endif

/* Context for fixed-block allocator */
typedef struct {
    MPID_THREAD_DS_LOCK_DECLARE  /* Lock variable for thread locking */
    MPIR_SBiAlloc *blocks;	 /* allocated storage */
    MPIR_SBblock  *avail;        /* fixed blocks (of size sizeb) to provide */
    int     nbfree, nballoc,     /* blocks free and in use */
            sizeb,               /* sizes in bytes */
            sizeincr;            /* # of blocks to allocate when more needed */
    } MPIR_SBHeader;

void MPIR_SBiAllocate();

void MPIR_SBdestroy(MPIR_SBHeader *sb);



int MPIR_HBT_lookup ( MPIR_HBT *tree, int keyval, MPIR_HBT_node **node );

int MPIR_HBT_new_node (int keyval, void *value, MPIR_HBT_node **node_out);

int MPIR_HBT_insert(MPIR_HBT *tree, MPIR_HBT_node *node);

int MPIR_HBT_new_tree (MPIR_HBT **tree_out);

int MPIR_HBT_free_tree (MPIR_HBT *tree);

int MPIR_HBT_delete(MPIR_HBT *tree, int keyval, MPIR_HBT_node **node_out);

int MPIR_HBT_free_node (MPIR_HBT_node *node);


int MPIR_Init(int  *argc, char ***argv);

int MPIR_Comm_init (MPI_Comm new_comm, MPI_Comm comm, MPIR_COMM_TYPE comm_type);

void MPIR_Comm_collops_init (MPI_Comm comm, MPIR_COMM_TYPE comm_type);

int MPIR_trvalid(char *str);


int MPIR_DoBufferSend(MPIR_SHANDLE *shandle);

void MPIR_BsendPersistent(MPI_Request request, int flag);


int MPIR_Pack2(char *buf, int count, MPI_Datatype type, int (*packcontig)(), 
			   void *packctx, char *dest, int *outlen, int *totlen);

int MPIR_Unpack2 (char *src, int count, MPI_Datatype type, int(*unpackcontig)(),
				  void *unpackctx, char *dest, int srclen, int *dest_len, int *used_len);


int MPID_WINNT_Test_send(MPIR_SHANDLE *dmpi_send_handle);

int MPID_WINNT_Test_recv_push(MPIR_RHANDLE *dmpi_recv_handle);

int MPID_SetPktSize(int len);

void MPID_SetDebugFlag(void *ctx, int f);

void MPID_SetMsgDebugFlag(void* ADIctx, int f);

void MPID_Set_tracefile(char *name);


int MPI_Type_size (MPI_Datatype  datatype, int *size);



