#include <crtdbg.h>
#include "windows.h"
#include "dm.h"
#include "packets.h"
#include "queue.h"
#include "queuedata.h"
#include "stdio.h"
#include "processinfo.h"
#include "infoprint.h"
#include "tchar.h"

DWORD WINAPI SHMEMReceiveThread(LPVOID lpV)
{
	DWORD i;
	MPID_WINNT_SHMEM_BUFFER_T *pShmemBuffer = NULL;
	extern HANDLE hEndProcessEvent;
	extern MPID_WINNT_SHMEM_BUFFER_T *ShmemBuffer[MAX_NUM_SHMEM_PROCS];
	extern WORLDPROCESSMAP *WorldProcessMap;
	HANDLE hEvents[2];
	DWORD dwStatus = ERROR_SUCCESS;
	LONG_PKT_HOLDING_CELL HoldingCell[MAX_WORLD_PROCS];
	DWORD dwBytesToCopy;
	DWORD dwSource;
	MPID_PKT_T *pShmemPkt = NULL;
	MPID_PKT_T *pTmpPkt = NULL;
	BOOL fStatus = FALSE;

	_ASSERTE(lpV);
	pShmemBuffer = (MPID_WINNT_SHMEM_BUFFER_T*)lpV;
	hEvents[0] = hEndProcessEvent;
	hEvents[1] = pShmemBuffer->hBufferFull;

	pShmemPkt = &(pShmemBuffer->Packet);
	for (i=0; i < MAX_WORLD_PROCS; i++)
	{
		HoldingCell[i].pPacket = NULL;
		HoldingCell[i].pCopyTo = NULL;
		HoldingCell[i].dwNumBytes = 0;
	}

	while (TRUE)
	{
		/*
			go ahead and allocate the packet here so we don't have
			to malloc it when receiving
		*/
		if (NULL == pTmpPkt)
			pTmpPkt = (MPID_PKT_T *)malloc(sizeof(MPID_PKT_T));
		_ASSERTE(pTmpPkt);
		if (NULL == pTmpPkt)
			goto reporterror;
		/* wait for an event to fire */
		dwStatus = WaitForMultipleObjects(2,hEvents,FALSE,INFINITE);
		_ASSERTE(WAIT_FAILED != dwStatus);
		if (WAIT_FAILED == dwStatus)
			goto reporterror;
		if (WAIT_OBJECT_0 == dwStatus)
			goto cleanup;
		/* it must be my shared memory buffer being full then */
		/* grab the mutex -- might want to wait for EndProcess ? */
		dwStatus = WaitForSingleObject(pShmemBuffer->hMutex,INFINITE);
		_ASSERTE(WAIT_FAILED != dwStatus);
		if (WAIT_FAILED == dwStatus)
			goto reporterror;
		/* well, now I have the mutex so I can get the data */
		switch (pShmemPkt->head.mode)
		{
		case MPID_PKT_SHORT:
			dwBytesToCopy =   sizeof(MPID_PKT_HEAD_T)
							+ pShmemPkt->head.len;
			memcpy(pTmpPkt,pShmemPkt,dwBytesToCopy);
			dwSource = pTmpPkt->head.src;
			Enqueue(RECEIVEQUEUE,pTmpPkt,dwSource,&dwStatus);
			pTmpPkt = NULL;
			break;
		case MPID_PKT_SHORT_SYNC:
			dwBytesToCopy =   sizeof(MPID_PKT_HEAD_T)
							+ pShmemPkt->head.len
							+ sizeof(DECL_SYNC_ID_T);
			memcpy(pTmpPkt,pShmemPkt,dwBytesToCopy);
			dwSource = pTmpPkt->head.src;
			Enqueue(RECEIVEQUEUE,pTmpPkt,dwSource,&dwStatus);
			pTmpPkt = NULL;
			break;
		case MPID_PKT_LONG:
			dwBytesToCopy = sizeof(MPID_PKT_LONG_T);
			memcpy(pTmpPkt,pShmemPkt,dwBytesToCopy);
			dwSource = pTmpPkt->head.src;
			pTmpPkt->long_pkt.pBuffer = (BYTE*)malloc(pTmpPkt->head.len);
			_ASSERTE(pTmpPkt->long_pkt.pBuffer);
			if (NULL == pTmpPkt->long_pkt.pBuffer)
				goto reporterror;
			_ASSERTE(NULL == HoldingCell[dwSource].pPacket);
			_ASSERTE(NULL == HoldingCell[dwSource].pCopyTo);
			_ASSERTE(0 == HoldingCell[dwSource].dwNumBytes);
			HoldingCell[dwSource].pPacket = pTmpPkt;
			HoldingCell[dwSource].pCopyTo = pTmpPkt->long_pkt.pBuffer;
			HoldingCell[dwSource].dwNumBytes = 0;
			pTmpPkt = NULL;
			break;
		case MPID_PKT_LONG_SYNC:
			dwBytesToCopy = sizeof(MPID_PKT_LONG_SYNC_T);
			memcpy(pTmpPkt,pShmemPkt,dwBytesToCopy);
			dwSource = pTmpPkt->head.src;
			pTmpPkt->long_sync_pkt.pBuffer = (BYTE*)malloc(pTmpPkt->head.len);
			_ASSERTE(pTmpPkt->long_sync_pkt.pBuffer);
			if (NULL == pTmpPkt->long_sync_pkt.pBuffer)
				goto reporterror;
			_ASSERTE(NULL == HoldingCell[dwSource].pPacket);
			_ASSERTE(NULL == HoldingCell[dwSource].pCopyTo);
			_ASSERTE(0 == HoldingCell[dwSource].dwNumBytes);
			HoldingCell[dwSource].pPacket = pTmpPkt;
			HoldingCell[dwSource].pCopyTo = pTmpPkt->long_sync_pkt.pBuffer;
			HoldingCell[dwSource].dwNumBytes = 0;
			pTmpPkt = NULL;
			break;
		case MPID_PKT_LONG_DATA:
			dwBytesToCopy = pShmemPkt->head.len;
			_ASSERTE(0 <= dwBytesToCopy);
			dwSource = pShmemPkt->head.src;
			_ASSERTE(HoldingCell[dwSource].pCopyTo);
			_ASSERTE(HoldingCell[dwSource].pPacket);
			memcpy(HoldingCell[dwSource].pCopyTo,pShmemPkt->long_data_pkt.buffer,dwBytesToCopy);
			HoldingCell[dwSource].pCopyTo += dwBytesToCopy;
			HoldingCell[dwSource].dwNumBytes += dwBytesToCopy;
			_ASSERTE(HoldingCell[dwSource].dwNumBytes <= (DWORD)(HoldingCell[dwSource].pPacket->head.len));
			if (HoldingCell[dwSource].dwNumBytes == (DWORD)(HoldingCell[dwSource].pPacket->head.len))
			{
				Enqueue(RECEIVEQUEUE,HoldingCell[dwSource].pPacket,dwSource,&dwStatus);
				HoldingCell[dwSource].pPacket = NULL;
				HoldingCell[dwSource].pCopyTo = NULL;
				HoldingCell[dwSource].dwNumBytes = 0;
			}
			break;
		case MPID_PKT_SYNC_ACK:
			dwBytesToCopy = sizeof(MPID_PKT_SYNC_ACK_T);
			memcpy(pTmpPkt,pShmemPkt,dwBytesToCopy);
			dwSource = pTmpPkt->head.src;
			Enqueue(RECEIVEQUEUE,pTmpPkt,dwSource,&dwStatus);
			pTmpPkt = NULL;
			break;
		}
		fStatus = SetEvent(pShmemBuffer->hBufferEmpty);
		_ASSERTE(fStatus);
		fStatus = ReleaseMutex(pShmemBuffer->hMutex);
		_ASSERTE(fStatus);
	}
cleanup:
	_ASSERTE(ERROR_SUCCESS == dwStatus);
	return dwStatus;
reporterror:
	dwStatus = GetLastError();
	goto cleanup;
}