#include "windows.h"
#include "packets.h"
#include "processinfo.h"
#include "infoprint.h"
#include "mpirun.h"
#include "crtdbg.h"

extern BOOL fExecutableSpecified;
extern BOOL fDebug;
extern DWORD dwNumProcs;
extern DWORD dwMyMachineID;
extern BOOL fWait;
extern WORD wPortNumber;
extern char szExecutable[MAXPATHLEN];
extern char szConfigFile[MAXPATHLEN];

DWORD SetUpWorldMap(WORLDPROCESSMAP *WorldProcessMap,JOBINFO *JobInfo)
{
	DWORD dwStatus = ERROR_SUCCESS;
	DWORD i,j;
	DWORD dwBufSize = 0;

	memset(WorldProcessMap,0,sizeof(WORLDPROCESSMAP));
	
	for (i=0; i < JobInfo->dwNumMachines; i++)
	{
		for (j=0; j < JobInfo->Machine[i].dwNumProcesses; j++)
		{
			WorldProcessMap->Process[JobInfo->Machine[i].Process[j].dwGlobalRank].dwMachineNumber = JobInfo->Machine[i].Process[j].dwMachineNumber;
			WorldProcessMap->Process[JobInfo->Machine[i].Process[j].dwGlobalRank].dwLocalBufferNumber = JobInfo->Machine[i].Process[j].dwLocalBufferNumber;
			WorldProcessMap->Process[JobInfo->Machine[i].Process[j].dwGlobalRank].eProtocol = E_SHMEM_PROTOCOL;
		}
	}
	WorldProcessMap->dwNumProcs = JobInfo->dwWorldSize;
	return dwStatus;
}

DWORD SetUpJobInfo(JOBINFO *JobInfo,int argc,char *argv[])
{
	DWORD dwStatus = ERROR_SUCCESS;
	DWORD i;
	DWORD dwBufSize = 0;

	memset(JobInfo,0,sizeof(JOBINFO));

	if (fExecutableSpecified)
	{
		dwMyMachineID = 0;
		JobInfo->dwNumMachines = 1;
		JobInfo->Machine[dwMyMachineID].dwNumProcesses = dwNumProcs;
		strcpy(JobInfo->Machine[dwMyMachineID].szMachineName,LOCALMACHINE_STR);
		JobInfo->dwMyMachineID = dwMyMachineID;
		for (i=0; i < JobInfo->Machine[dwMyMachineID].dwNumProcesses; i++)
		{
			strcpy(JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].szExecutable,szExecutable);
			JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].dwLocalBufferNumber = i;
			JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].dwMachineNumber = JobInfo->dwMyMachineID;
			JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].dwGlobalRank = i;
		}
		JobInfo->dwWorldSize = 0;
		for (i=0; i < JobInfo->dwNumMachines; i++)
			JobInfo->dwWorldSize += JobInfo->Machine[i].dwNumProcesses;
	}
	else
	{
		BOOL fStatus = FALSE;
		extern BOOL ReadConfigFile(char *szConfigFile,JOBINFO *JobInfo);
		fStatus = ReadConfigFile(szConfigFile,JobInfo);
		if (!fStatus)
			dwStatus = GetLastError();
		PrintJobInfo(JobInfo);
	}
	if (ERROR_SUCCESS == dwStatus)
		JobInfo->wPortNumber = 12397;
	return dwStatus;
}

DWORD RunMPIJob(int argc,char *argv[])
{
	DWORD dwStatus = ERROR_SUCCESS;
	JOBINFO MemJobInfo;
	WORLDPROCESSMAP MemWorldProcessMap;
	char szMyName[MACHINENAMELEN];


	dwStatus = SetUpJobInfo(&MemJobInfo,argc,argv);
	_ASSERTE(ERROR_SUCCESS == dwStatus);
	if (ERROR_SUCCESS != dwStatus)
		goto reporterror;

	if (fDebug)
		PrintJobInfo(&MemJobInfo);

	dwStatus = SetUpWorldMap(&MemWorldProcessMap,&MemJobInfo);
	_ASSERTE(ERROR_SUCCESS == dwStatus);
	if (ERROR_SUCCESS != dwStatus)
		goto reporterror;

	if (fDebug)
		PrintWorldProcessMap(&MemWorldProcessMap);


	/*
		Now that I have the Job information, figure out if I
		can run it local or not.
	*/

	dwStatus = (DWORD)gethostname(szMyName,MACHINENAMELEN);

	if (0 == dwStatus)
	{
		if ((1 == MemJobInfo.dwNumMachines))
		{
			
			if (!strcmp(LOCALMACHINE_STR,MemJobInfo.Machine[0].szMachineName))
				dwStatus = MPIRunStartMPIJob(&MemJobInfo,&MemWorldProcessMap);
			else if (!strcmp(szMyName,MemJobInfo.Machine[0].szMachineName))
				dwStatus = MPIRunStartMPIJob(&MemJobInfo,&MemWorldProcessMap);
			else
				dwStatus = RunRemoteMPIJob(&MemJobInfo,&MemWorldProcessMap);
		}
		else
		{
			dwStatus = RunRemoteMPIJob(&MemJobInfo,&MemWorldProcessMap);
		}
	}
	else
		goto reporterror;

cleanup:
	return dwStatus;

reporterror:
	dwStatus = GetLastError();
	printf("\nExitting with status [%d]\n",dwStatus);
	goto cleanup;
}