//  MODULE:   CreateRegistryInfo.c
//
//  PURPOSE:  Creates the registry information needed by the service
//
//  FUNCTIONS:
//
//		LONG CreateRegistryInfo(int argc,char **argv)
//
//  COMMENTS:
//
//  AUTHOR: Shane Hebert
//


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <winsock.h>
#include "service.h"



//
//  FUNCTION: CreateRegistryInfo()
//
//  PURPOSE: sets up the registry info for the service
//			initially, this is the port number to listen
//			to and possibly the version number
//
//  PARAMETERS:
//		NONE
//
//  RETURN VALUE:
//    status
//
//  COMMENTS:
//
LONG CreateRegistryInfo(int argc,char **argv)
{
	HKEY hkSoftwareSubKey;
	HKEY hkMPISubKey;
	DWORD dwOptions = REG_OPTION_NON_VOLATILE;
	REGSAM samDesired = KEY_ALL_ACCESS;
	SECURITY_ATTRIBUTES saSecurity;
	PSECURITY_DESCRIPTOR pSD;
	DWORD dwDisposition;
	LONG nStatus;
	DWORD dwPortNumber=12396;
	LPTSTR lpszSoftwareSubKey = TEXT(SZSOFTWARESUBKEY);
	LPTSTR lpszMPISubKey = TEXT(SZMPISUBKEY);
	LPTSTR lpszKeyClass = TEXT(SZKEYCLASS);
	LPTSTR lpszVersionText = TEXT(SZVERSIONTEXT);
	LPTSTR lpszVersionNumber = TEXT(SZVERSIONNUMBER);
	LPTSTR lpszPortNumberText = TEXT(SZPORTNUMBERTEXT);
	int i,found;

	if (2 > argc)
		goto cleanup;
	
	// find the parameter designating the port to
	// listen at
	//
	for (found=0,i=1;i<argc;i++)
	{
		if (argv[i][1] == 'p')
		{
			found = 1;
			break;
		}
	}

	// we found it so use it.
	if (found && (i < argc))
		dwPortNumber = (WORD)atoi(&(argv[i][2]));

	// We are going to store the key in HKEY_LOCAL_MACHINE
	// so open it (it always exists)
	nStatus = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			lpszSoftwareSubKey,
			0,
			KEY_ALL_ACCESS,
			&hkSoftwareSubKey);
	if (ERROR_SUCCESS == nStatus) {
		// Now we have to make security attributes.
		// for now, we will use null security (open to
		// everyone.
		pSD = (PSECURITY_DESCRIPTOR)malloc(SECURITY_DESCRIPTOR_MIN_LENGTH);
		if (NULL == pSD)
			goto cleanup;
		if (!InitializeSecurityDescriptor(pSD,SECURITY_DESCRIPTOR_REVISION))
			goto cleanup;
		// add a NULL disc. ACL to the security descriptor.
		//
		if (!SetSecurityDescriptorDacl(pSD, TRUE, (PACL) NULL, FALSE))
			goto cleanup;
		saSecurity.nLength = sizeof(SECURITY_DESCRIPTOR);
		saSecurity.lpSecurityDescriptor = pSD;
		saSecurity.bInheritHandle = TRUE;
	}
	if (ERROR_SUCCESS == nStatus)
	{
		//  Create will open the key if already present.
		//  if the MPI Server is already installed, we
		//	should never get here anyway but just in case...
		//  Anyway, this will create the key "MPI Server"
		nStatus = RegCreateKeyEx(
			hkSoftwareSubKey,
			lpszMPISubKey,
			0,
			lpszKeyClass,
			dwOptions,
			samDesired,
			&saSecurity,
			&hkMPISubKey,
			&dwDisposition);
		if (ERROR_SUCCESS == nStatus)
		{
			switch (dwDisposition)
			{
			case REG_OPENED_EXISTING_KEY:
				//  If the key existed prior to this, clean
				//  it out (shouldn't happen
				RegDeleteValue(hkMPISubKey,lpszPortNumberText);
				RegDeleteValue(hkMPISubKey,lpszVersionText);
			case REG_CREATED_NEW_KEY:
				// Set the value that contains the Port Number
				nStatus = RegSetValueEx(
								hkMPISubKey,
								lpszPortNumberText,
								0,
								(DWORD)REG_DWORD,
								(BYTE*)&dwPortNumber,
								sizeof(DWORD));
				if (ERROR_SUCCESS == nStatus)
				{
					int nLength = strlen(lpszVersionNumber)+1;
					//  I couldn't think of anything else
					//  right now so I'll store the version
					//  as well
					nStatus = RegSetValueEx(
								hkMPISubKey,
								lpszVersionText,
								0,
								(DWORD)REG_SZ,
								lpszVersionNumber,
								nLength);
				}
				break;
			default:
				break;
			}
		}
	}
cleanup:
	if (ERROR_SUCCESS != nStatus)
	{
		TCHAR szMsg[512];
		GetLastErrorText(szMsg,512);
		AddToMessageLog(szMsg);
	}
	else
	{
		RegCloseKey(hkMPISubKey);
		RegCloseKey(hkSoftwareSubKey);
	}
	if (pSD)
		free(pSD);
	return nStatus;
}
