//  MODULE:   MPIServeCreateSingleProcess.c
//
//  PURPOSE:	Function to create a single process
//				(not yet defined)
//
//  FUNCTIONS:
//
//		DWORD WINAPI MPIServeCreateSingleProcess(LPVOID lpV)
//
//  COMMENTS:
//
//  AUTHOR: Shane Hebert
//


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <winsock.h>
#include "service.h"
#include "processinfo.h"
#include "mpiserve.h"
#include "SocketReadAndWrite.h"
#include "MPIServePacket.h"

DWORD WINAPI MPIServeCreateSingleProcess(LPVOID lpV)
{
	DWORD dwStatus = ERROR_SUCCESS;
	BOOL fStatus = FALSE;
	SOCKET sSocket = (SOCKET)lpV;
	MPIServeCreateSingleProcessPacket_T ProcPacket;
	MPIServeCreateSingleProcessReturnDataPacket_T ReturnPacket;
	SECURITY_ATTRIBUTES sa;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	
	dwStatus = RecvSocketN(sSocket,
							(BYTE*)&ProcPacket,
							sizeof(MPIServeCreateSingleProcessPacket_T));
	if (SOCKET_ERROR != dwStatus)
		dwStatus = ERROR_SUCCESS;
	else
		goto reporterror;

	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = FALSE;

	ReturnPacket.dwProcessID = 0;
	ReturnPacket.hProcess = 0;
	fStatus = CreateProcess(NULL,							/* application name */
							ProcPacket.szExecutableName,	/* command line */
							&sa,							/* process SA */
							&sa,							/* thread SA */
							FALSE,							/* inherit flag */
							0,								/* creation flags */
							NULL,							/* environment block */
							"\\",							/* working directory */
							&si,							/* startup info */
							&pi);

	ReturnPacket.dwProcessExitStatus = 0;
	if (fStatus)
	{
		ReturnPacket.dwProcessID = pi.dwProcessId;
		ReturnPacket.hProcess = pi.hProcess;
	}
	else
		dwStatus = GetLastError();

	if (!ProcPacket.fBlocking || (ERROR_SUCCESS != dwStatus))
	{
		ReturnPacket.dwStatus = dwStatus;
		SendSocketN(sSocket,(BYTE*)&ReturnPacket,sizeof(MPIServeCreateSingleProcessReturnDataPacket_T));
		closesocket(sSocket);
		goto cleanup;
	}

	dwStatus = WaitForSingleObject(pi.hProcess,INFINITE);
	if (WAIT_FAILED != dwStatus)
		fStatus = GetExitCodeProcess(pi.hProcess,&ReturnPacket.dwProcessExitStatus);
	else
		ReturnPacket.dwStatus = dwStatus;

	if (ProcPacket.fBlocking)
	{
		ReturnPacket.dwStatus = dwStatus;
		SendSocketN(sSocket,(BYTE*)&ReturnPacket,sizeof(MPIServeCreateSingleProcessReturnDataPacket_T));
		closesocket(sSocket);
	}
cleanup:
	if (pi.hProcess)
		fStatus = CloseHandle(pi.hProcess);
	if (pi.hThread)
		fStatus = CloseHandle(pi.hThread);
	return dwStatus;
reporterror:
	dwStatus = WSAGetLastError();
	goto cleanup;
}
