//  MODULE:   MPIServeJobQueue.c
//
//  PURPOSE:	Queues the MPI Jobs for job control
//
//  FUNCTIONS:
//
//  COMMENTS:
//
//  AUTHOR: Shane Hebert
//


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <winsock.h>
#include "service.h"
#include "processinfo.h"
#include "mpiserve.h"
#include "mpiservepacket.h"
#include "packets.h"

static JOBQUEUE *JobQueue = NULL;
static BOOL fQueueInitialized = FALSE;
static CRITICAL_SECTION csJobQueue;

static BOOL InitializeJobQueue()
{
	InitializeCriticalSection(&csJobQueue);
	fQueueInitialized = TRUE;
	return fQueueInitialized;
}

DWORD Enqueue(JOBQUEUE *Job)
{
	DWORD dwStatus = ERROR_SUCCESS;
	JOBQUEUE *pJQ;

	if (!fQueueInitialized)
	{
		InitializeJobQueue();
		if (!fQueueInitialized)
			goto cleanup;
	}
	EnterCriticalSection(&csJobQueue);
	pJQ = JobQueue;
	JobQueue = pJQ;
	LeaveCriticalSection(&csJobQueue);
cleanup:
	return dwStatus;
}

DWORD Dequeue(char szJobID[],JOBQUEUE **Job)
{
	DWORD dwStatus = ERROR_SUCCESS;
	JOBQUEUE *pJQ,*p2JQ;
	BOOL fFound = FALSE;

	p2JQ = NULL;
	*Job = NULL;
	EnterCriticalSection(&csJobQueue);
	pJQ = JobQueue;
	for (pJQ = JobQueue,p2JQ = NULL; pJQ ; pJQ = pJQ->pNext)
	{
		if (!strcmp(szJobID,pJQ->szJobID))
		{
			if (p2JQ)
			{
				p2JQ->pNext = pJQ->pNext;
				pJQ->pNext = NULL;
			}
			else
			{
				/* gotta be the head node */
				JobQueue = pJQ->pNext;
				pJQ->pNext = NULL;
			}
			*Job = pJQ;
			fFound = TRUE;
			break;
		}
		p2JQ = pJQ;
	}
	LeaveCriticalSection(&csJobQueue);
	if (fFound)
		dwStatus = ERROR_SUCCESS;
	else
		dwStatus = (DWORD)(~ERROR_SUCCESS);
	return dwStatus;
}
