#include "windows.h"
#include "packets.h"
#include "processinfo.h"
#include "infoprint.h"
#include "crtdbg.h"


DWORD RunLocalMPIJobProcesses(JOBINFO *MemJI,WORLDPROCESSMAP *MemWPM)
{
	DWORD dwStatus = ERROR_SUCCESS;
	HANDLE hJobInfoFileMap = NULL;
	HANDLE hWorldProcessMapFileMap = NULL;
	HANDLE hLocalProcessHandles = NULL;
	LOCALPROCESSHANDLES *LocalProcessHandles = NULL;
	WORLDPROCESSMAP *WorldProcessMap = NULL;
	JOBINFO *JobInfo = NULL;
	SECURITY_ATTRIBUTES sa;
	STARTUPINFO si;
	PROCESS_INFORMATION piProcess[MAX_NUM_SHMEM_PROCS];
	DWORD dwBufSize = 0;
	MPID_WINNT_SHMEM_BUFFER_T *pBuf;
	DWORD i;

	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = TRUE;

	hLocalProcessHandles = CreateFileMapping((HANDLE) 0xFFFFFFFF,	/* create a new one */
											&sa,					/* null security attributes */
											PAGE_READWRITE | SEC_COMMIT,	/* read and write to the memory */
											0,						/* high order bits */
											sizeof(LOCALPROCESSHANDLES),	/* low order bits */
											NULL);					/* no name */
	_ASSERTE(hLocalProcessHandles);
	if (NULL == hLocalProcessHandles)
		goto reporterror;

	LocalProcessHandles = (LOCALPROCESSHANDLES *)
							MapViewOfFile(hLocalProcessHandles,	/* handle to file mapping */
										FILE_MAP_ALL_ACCESS,		/* all access */
										0,							/* high order bits */
										0,							/* low order bits */
										0);							/* map the whole file */
	_ASSERTE(LocalProcessHandles);
	if (NULL == LocalProcessHandles)
		goto reporterror;

	
	
	hWorldProcessMapFileMap = CreateFileMapping((HANDLE) 0xFFFFFFFF,	/* create a new one */
												&sa,					/* null security attributes */
												PAGE_READWRITE,			/* read and write to the memory */
												0,						/* high order bits */
												sizeof(WORLDPROCESSMAP),/* low order bits */
												NULL);					/* no name */
	_ASSERTE(hWorldProcessMapFileMap);
	if (NULL == hWorldProcessMapFileMap)
		goto reporterror;

	WorldProcessMap = (WORLDPROCESSMAP *) MapViewOfFile(hWorldProcessMapFileMap,	/* handle to file mapping */
														FILE_MAP_ALL_ACCESS,		/* all access */
														0,							/* high order bits */
														0,							/* low order bits */
														0);							/* map the whole file */
	_ASSERTE(WorldProcessMap);
	if (NULL == WorldProcessMap)
		goto reporterror;

	hJobInfoFileMap = CreateFileMapping((HANDLE) 0xFFFFFFFF,	/* create a new one */
										&sa,					/* null security attributes */
										PAGE_READWRITE | SEC_COMMIT,			/* read and write to the memory */
										0,						/* high order bits */
										sizeof(JOBINFO),		/* low order bits */
										NULL);					/* no name */
	_ASSERTE(hJobInfoFileMap);
	if (NULL == hJobInfoFileMap)
		goto reporterror;

	JobInfo = (JOBINFO *) MapViewOfFile(hJobInfoFileMap,	/* handle to file mapping */
										FILE_MAP_ALL_ACCESS,		/* all access */
										0,							/* high order bits */
										0,							/* low order bits */
										0);							/* map the whole file */
	_ASSERTE(JobInfo);
	if (NULL == JobInfo)
		goto reporterror;

	/*
		now that I have shared memory to write all the job information to,
		I can fill out the job information stuff
	*/

	/* copy over everything sent to me into shared memory */
	memcpy(JobInfo,MemJI,sizeof(JOBINFO));
	memcpy(WorldProcessMap,MemWPM,sizeof(WORLDPROCESSMAP));

	/* now get all the events, mutexes, etc */

	if (1 < JobInfo->Machine[JobInfo->dwMyMachineID].dwNumProcesses)
	{
		JobInfo->hBarrierSemaphore = CreateSemaphore(&sa,				/* security */
													 JobInfo->Machine[JobInfo->dwMyMachineID].dwNumProcesses - 1,	/* initial count */
													 JobInfo->Machine[JobInfo->dwMyMachineID].dwNumProcesses - 1,	/* max count */
													 NULL);				/* no name */
		_ASSERTE(JobInfo->hBarrierSemaphore);
		if (NULL == JobInfo->hBarrierSemaphore)
		{
			printf("Unable to create Barrier Semaphore!  Exitting...\n");
			goto reporterror;
		}
		JobInfo->hBarrierEvent = CreateEvent(&sa,	/* security */
											 TRUE,	/* manual reset */
											 FALSE, /* initially signalled? */
											 NULL); /* no name */
		_ASSERTE(JobInfo->hBarrierEvent);
		if (NULL == JobInfo->hBarrierEvent)
		{
			printf("Unable to create Barrier Event!  Exitting...\n");
			goto reporterror;
		}
	}

	JobInfo->hGlobalTerminatorEvent = CreateEvent(&sa,		/* security */
												  TRUE,		/* manual reset */
												  FALSE,	/* initally signalled? */
												  NULL);	/* no name */
	_ASSERTE(JobInfo->hGlobalTerminatorEvent);
	if (NULL == JobInfo->hGlobalTerminatorEvent)
	{
		printf("Unable to create Global Terminator Event!  Exitting...\n");
		goto reporterror;
	}

	dwBufSize = sizeof(MPID_WINNT_SHMEM_BUFFER_T);
	for (i=0; i < JobInfo->Machine[JobInfo->dwMyMachineID].dwNumProcesses; i++)
	{
		JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].hSharedMemoryBuffer = CreateFileMapping((HANDLE) 0xFFFFFFFF,		/* create a new one */
																									&sa,						/* null security attributes */
																									PAGE_READWRITE | SEC_NOCACHE | SEC_COMMIT,	/* read and write to the memory */
																									0,						/* high order bits */
																									dwBufSize,				/* low order bits */
																									NULL);					/* no name */
		_ASSERTE(JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].hSharedMemoryBuffer);
		if (NULL == JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].hSharedMemoryBuffer)
		{
			printf("Unable to create Shared Memory Buffer for Message Passing!  Exitting...\n");
			goto reporterror;
		}

		pBuf = (MPID_WINNT_SHMEM_BUFFER_T *)MapViewOfFile(JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].hSharedMemoryBuffer, /* handle to file mapping */
														  FILE_MAP_ALL_ACCESS,											   /* all access */
														  0,															   /* high order bits */
														  0,															   /* low order bits */
														  0);															   /* map the whole file */
		_ASSERTE(pBuf);
		if (NULL == pBuf)
		{
			printf("Unable to Map View of Shared Memory Buffer!  Exitting...\n");
			goto reporterror;
		}
		pBuf->hBufferEmpty = CreateEvent(&sa,
										 FALSE,
										 TRUE,
										 NULL);
		_ASSERTE(pBuf->hBufferEmpty);
		if (NULL == pBuf->hBufferEmpty)
			goto reporterror;
		pBuf->hBufferFull = CreateEvent(&sa,
										FALSE,
										FALSE,
										NULL);
		_ASSERTE(pBuf->hBufferFull);
		if (NULL == pBuf->hBufferFull)
			goto reporterror;

		pBuf->hMutex = CreateMutex(&sa,
								   FALSE,
								   NULL);
		_ASSERTE(pBuf->hMutex);
		if (NULL == pBuf->hMutex)
			goto reporterror;

		UnmapViewOfFile(pBuf);
	}
	
	
	ZeroMemory(&si, sizeof(si));
	si.cb = sizeof(si);

	printf("\n");

	for (i = 0; i < MAX_NUM_SHMEM_PROCS; i++)
	{
		LocalProcessHandles->hProc[i] = (HANDLE)NULL;
		LocalProcessHandles->hThread[i] = (HANDLE)NULL;
	}

	LocalProcessHandles->dwNumHandles = JobInfo->Machine[JobInfo->dwMyMachineID].dwNumProcesses;
	for (i=0; i < JobInfo->Machine[JobInfo->dwMyMachineID].dwNumProcesses; i++)
	{
		BOOL fStatus = TRUE;
		char szArgs[MAXPATHLEN];

		sprintf(szArgs," %s %d %s %d %s %d %s %d",
					   CMD_GLOBALRANK_STR,
					   JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].dwGlobalRank,
					   CMD_JOBINFO_STR,
					   hJobInfoFileMap,
					   CMD_WORLDMAP_STR,
					   hWorldProcessMapFileMap,
					   CMD_PROCESS_HANDLES_STR,
					   hLocalProcessHandles);
		strcat(JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].szExecutable,szArgs);
		
		fStatus = CreateProcess(NULL,
								JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].szExecutable,
								&sa,
								&sa,
								TRUE,
								CREATE_SUSPENDED,
								NULL,
								NULL,
								&si,
								&(piProcess[i]));
			
		_ASSERTE(fStatus);
		if (FALSE == fStatus)
			goto abortprocesses;
		_ASSERTE(piProcess[i].hProcess);
		LocalProcessHandles->hProc[i] = piProcess[i].hProcess;
		_ASSERTE(piProcess[i].hThread);
		LocalProcessHandles->hThread[i] = piProcess[i].hThread;
		/* hProcess[i] = piProcess[i].hProcess; */
		if (fDebug)
			printf("Created Process [%s] with ID [0x%x]...\n",JobInfo->Machine[JobInfo->dwMyMachineID].Process[i].szExecutable,piProcess[i].dwProcessId);
	}

	for (i=0; i < LocalProcessHandles->dwNumHandles; i++)
	{
		dwStatus = ResumeThread(LocalProcessHandles->hThread[i]);
		if (0xFFFFFFFF == dwStatus)
			goto abortprocesses;
	}

	dwStatus = WaitForMultipleObjects(LocalProcessHandles->dwNumHandles,
									  LocalProcessHandles->hProc,
									  TRUE,
									  INFINITE);
	if (WAIT_FAILED == dwStatus)
		goto reporterror;

cleanup:
	if (fDebug)
	{
		printf("\n");
		for (i=0;i < LocalProcessHandles->dwNumHandles;i++)
		{
			BOOL fStatus;
			DWORD dwExitCode;

			if (LocalProcessHandles->hProc[i])
			{
				fStatus = GetExitCodeProcess(LocalProcessHandles->hProc[i],&dwExitCode);
				if (fStatus)
					printf("Process [0x%x] exitted with code [%d]\n",
							piProcess[i].dwProcessId,dwExitCode);
				else
					printf("Unable to get process [0x%x] exit code\n",
							piProcess[i].dwProcessId);
			}
		}
	}
	if (LocalProcessHandles)
	{
		for (i=0; i < LocalProcessHandles->dwNumHandles; i++)
		{
			if (LocalProcessHandles->hProc[i])
			{
				CloseHandle(LocalProcessHandles->hProc[i]);
				LocalProcessHandles->hProc[i] = NULL;
			}
			if (LocalProcessHandles->hThread[i])
			{
				CloseHandle(LocalProcessHandles->hThread[i]);
				LocalProcessHandles->hThread[i] = NULL;
			}
		}
		UnmapViewOfFile(LocalProcessHandles);
	}
	if (hLocalProcessHandles)
		CloseHandle(hLocalProcessHandles);
	if (WorldProcessMap)
	{
		FreeWorldProcessMap(WorldProcessMap);
		UnmapViewOfFile(WorldProcessMap);
	}
	if (hWorldProcessMapFileMap)
		CloseHandle(hWorldProcessMapFileMap);
	if (JobInfo)
	{
		FreeJobInfo(JobInfo);
		UnmapViewOfFile(JobInfo);
	}
	if (hJobInfoFileMap)
		CloseHandle(hJobInfoFileMap);
	return dwStatus;

reporterror:
	dwStatus = GetLastError();
	printf("\nExitting with status [%d]\n",dwStatus);
	goto cleanup;

abortprocesses:
	dwStatus = GetLastError();
	for (i=0;i < MAX_NUM_SHMEM_PROCS; i++)
	{
		BOOL fStatus;
		if (LocalProcessHandles->hProc[i])
		{
			fStatus = TerminateProcess(LocalProcessHandles->hProc[i],(UINT)0xFFFFFFFF);
			_ASSERTE(fStatus);
			if (FALSE == fStatus)
				printf("Cannot kill process with handle [%d]!\n",
						LocalProcessHandles->hProc[i]);
		}
	}
	goto reporterror;
}